@@ -1,6 +1,12 @@
 import sys,os
-from COMTool import parameters,helpAbout,autoUpdate
-from COMTool.Combobox import ComboBox
+try:
+    import parameters,helpAbout,autoUpdate
+    from Combobox import ComboBox
+except ImportError:
+    from COMTool import parameters,helpAbout,autoUpdate
+    from COMTool.Combobox import ComboBox
+
+# from COMTool.wave import Wave
 from PyQt5.QtCore import pyqtSignal,Qt
 from PyQt5.QtWidgets import (QApplication, QWidget,QToolTip,QPushButton,QMessageBox,QDesktopWidget,QMainWindow,
                              QVBoxLayout,QHBoxLayout,QGridLayout,QTextEdit,QLabel,QRadioButton,QCheckBox,
@@ -8,7 +14,6 @@ from PyQt5.QtWidgets import (QApplication, QWidget,QToolTip,QPushButton,QMessage
 from PyQt5.QtGui import QIcon,QFont,QTextCursor,QPixmap
 import serial
 import serial.tools.list_ports
-import serial.threaded
 import threading
 import time
 import binascii,re
@@ -19,17 +24,25 @@ except ImportError:
 if sys.platform == "win32":
     import ctypes
 
+class MyClass(object):
+    def __init__(self, arg):
+        super(MyClass, self).__init__()
+        self.arg = arg
+
 class MainWindow(QMainWindow):
     receiveUpdateSignal = pyqtSignal(str)
     errorSignal = pyqtSignal(str)
+    showSerialComboboxSignal = pyqtSignal()
+    setDisableSettingsSignal = pyqtSignal(bool)
     isDetectSerialPort = False
     receiveCount = 0
     sendCount = 0
     isScheduledSending = False
     DataPath = "./"
     isHideSettings = False
-    isHideFunctinal = False
+    isHideFunctinal = True
     app = None
+    isWaveOpen = False
 
     def __init__(self,app):
         super().__init__()
@@ -45,13 +58,12 @@ class MainWindow(QMainWindow):
         self.initTool()
         self.initEvent()
         self.programStartGetSavedParameters()
-        return
 
     def __del__(self):
-        return
+        pass
+
     def initTool(self):
         self.com = serial.Serial()
-        return
 
     def initWindow(self):
         QToolTip.setFont(QFont('SansSerif', 10))
@@ -86,20 +98,32 @@ class MainWindow(QMainWindow):
         # option layout
         self.settingsButton = QPushButton()
         self.skinButton = QPushButton("")
+        # self.waveButton = QPushButton("")
         self.aboutButton = QPushButton()
         self.functionalButton = QPushButton()
+        self.encodingCombobox = ComboBox()
+        self.encodingCombobox.addItem("ASCII")
+        self.encodingCombobox.addItem("UTF-8")
+        self.encodingCombobox.addItem("UTF-16")
+        self.encodingCombobox.addItem("GBK")
+        self.encodingCombobox.addItem("GB2312")
+        self.encodingCombobox.addItem("GB18030")
         self.settingsButton.setProperty("class", "menuItem1")
         self.skinButton.setProperty("class", "menuItem2")
         self.aboutButton.setProperty("class", "menuItem3")
         self.functionalButton.setProperty("class", "menuItem4")
+        # self.waveButton.setProperty("class", "menuItem5")
         self.settingsButton.setObjectName("menuItem")
         self.skinButton.setObjectName("menuItem")
         self.aboutButton.setObjectName("menuItem")
         self.functionalButton.setObjectName("menuItem")
+        # self.waveButton.setObjectName("menuItem")
         menuLayout.addWidget(self.settingsButton)
         menuLayout.addWidget(self.skinButton)
+        # menuLayout.addWidget(self.waveButton)
         menuLayout.addWidget(self.aboutButton)
         menuLayout.addStretch(0)
+        menuLayout.addWidget(self.encodingCombobox)
         menuLayout.addWidget(self.functionalButton)
 
 
@@ -162,6 +186,8 @@ class MainWindow(QMainWindow):
         self.serailStopbitsCombobox.addItem("1.5")
         self.serailStopbitsCombobox.addItem("2")
         self.serailStopbitsCombobox.setCurrentIndex(0)
+        self.checkBoxRts = QCheckBox("rts")
+        self.checkBoxDtr = QCheckBox("dtr")
         self.serialOpenCloseButton = QPushButton(parameters.strOpen)
         serialSettingsLayout.addWidget(serialPortLabek,0,0)
         serialSettingsLayout.addWidget(serailBaudrateLabel, 1, 0)
@@ -173,7 +199,9 @@ class MainWindow(QMainWindow):
         serialSettingsLayout.addWidget(self.serailBytesCombobox, 2, 1)
         serialSettingsLayout.addWidget(self.serailParityCombobox, 3, 1)
         serialSettingsLayout.addWidget(self.serailStopbitsCombobox, 4, 1)
-        serialSettingsLayout.addWidget(self.serialOpenCloseButton, 5, 0,1,2)
+        serialSettingsLayout.addWidget(self.checkBoxRts, 5, 0,1,1)
+        serialSettingsLayout.addWidget(self.checkBoxDtr, 5, 1,1,1)
+        serialSettingsLayout.addWidget(self.serialOpenCloseButton, 6, 0,1,2)
         serialSettingsGroupBox.setLayout(serialSettingsLayout)
         settingLayout.addWidget(serialSettingsGroupBox)
 
@@ -223,6 +251,8 @@ class MainWindow(QMainWindow):
         functionalGridLayout.addWidget(self.addButton,0,1)
         functionalGroupBox.setLayout(functionalGridLayout)
         sendFunctionalLayout.addWidget(functionalGroupBox)
+        self.isHideFunctinal = True
+        self.hideFunctional()
 
         # main window
         self.statusBarStauts = QLabel()
@@ -245,7 +275,7 @@ class MainWindow(QMainWindow):
         if sys.platform == "win32":
             ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID("comtool")
         self.show()
-        return
+        print("config file path:",os.getcwd()+"/comtool.settings.config")
 
     def initEvent(self):
         self.serialOpenCloseButton.clicked.connect(self.openCloseSerial)
@@ -256,27 +286,38 @@ class MainWindow(QMainWindow):
         self.sendSettingsHex.clicked.connect(self.onSendSettingsHexClicked)
         self.sendSettingsAscii.clicked.connect(self.onSendSettingsAsciiClicked)
         self.errorSignal.connect(self.errorHint)
+        self.showSerialComboboxSignal.connect(self.showCombobox)
+        # self.showBaudComboboxSignal.connect(self.showBaudCombobox)
+        self.setDisableSettingsSignal.connect(self.setDisableSettings)
         self.sendHistory.currentIndexChanged.connect(self.sendHistoryIndexChanged)
         self.settingsButton.clicked.connect(self.showHideSettings)
         self.skinButton.clicked.connect(self.skinChange)
         self.aboutButton.clicked.connect(self.showAbout)
         self.addButton.clicked.connect(self.functionAdd)
         self.functionalButton.clicked.connect(self.showHideFunctional)
-        return
+        self.sendArea.currentCharFormatChanged.connect(self.sendAreaFontChanged)
+        # self.waveButton.clicked.connect(self.openWaveDisplay)
+        self.checkBoxRts.clicked.connect(self.rtsChanged)
+        self.checkBoxDtr.clicked.connect(self.dtrChanged)
+
+        self.myObject=MyClass(self)
+        slotLambda = lambda: self.indexChanged_lambda(self.myObject)
+        self.serialPortCombobox.currentIndexChanged.connect(slotLambda)
+
+
+    # @QtCore.pyqtSlot(str)
+    def indexChanged_lambda(self, obj):
+        mainObj = obj.arg
+        # print("item changed:",mainObj.serialPortCombobox.currentText())
+        self.serialPortCombobox.setToolTip(mainObj.serialPortCombobox.currentText())
+
 
     def openCloseSerialProcess(self):
         try:
             if self.com.is_open:
-                self.com.close()
-                self.serialOpenCloseButton.setText(parameters.strOpen)
-                self.statusBarStauts.setText("<font color=%s>%s</font>" % ("#f31414", parameters.strClosed))
                 self.receiveProgressStop = True
-                self.serialPortCombobox.setDisabled(False)
-                self.serailBaudrateCombobox.setDisabled(False)
-                self.serailParityCombobox.setDisabled(False)
-                self.serailStopbitsCombobox.setDisabled(False)
-                self.serailBytesCombobox.setDisabled(False)
-                self.programExitSaveParameters()
+                self.com.close()
+                self.setDisableSettingsSignal.emit(False)
             else:
                 try:
                     self.com.baudrate = int(self.serailBaudrateCombobox.currentText())
@@ -285,38 +326,69 @@ class MainWindow(QMainWindow):
                     self.com.parity = self.serailParityCombobox.currentText()[0]
                     self.com.stopbits = float(self.serailStopbitsCombobox.currentText())
                     self.com.timeout = None
-                    print(self.com)
+                    if self.checkBoxRts.isChecked():
+                        self.com.rts = False
+                    else:
+                        self.com.rts = True
+                    if self.checkBoxDtr.isChecked():
+                        self.com.dtr = False
+                    else:
+                        self.com.dtr = True
                     self.serialOpenCloseButton.setDisabled(True)
                     self.com.open()
-                    self.serialOpenCloseButton.setText(parameters.strClose)
-                    self.statusBarStauts.setText("<font color=%s>%s</font>" % ("#008200", parameters.strReady))
-                    self.serialPortCombobox.setDisabled(True)
-                    self.serailBaudrateCombobox.setDisabled(True)
-                    self.serailParityCombobox.setDisabled(True)
-                    self.serailStopbitsCombobox.setDisabled(True)
-                    self.serailBytesCombobox.setDisabled(True)
-                    self.serialOpenCloseButton.setDisabled(False)
-                    receiveProcess = threading.Thread(target=self.receiveData)
-                    receiveProcess.setDaemon(True)
-                    receiveProcess.start()
+                    print("open success")
+                    print(self.com)
+                    self.setDisableSettingsSignal.emit(True)
+                    self.receiveProcess = threading.Thread(target=self.receiveData)
+                    self.receiveProcess.setDaemon(True)
+                    self.receiveProcess.start()
                 except Exception as e:
                     self.com.close()
                     self.receiveProgressStop = True
                     self.errorSignal.emit( parameters.strOpenFailed +"\n"+ str(e))
                     self.serialOpenCloseButton.setDisabled(False)
-        except Exception:
-            pass
-        return
+        except Exception as e:
+            print(e)
+    
+    def setDisableSettings(self, disable):
+        if disable:
+            self.serialOpenCloseButton.setText(parameters.strClose)
+            self.statusBarStauts.setText("<font color=%s>%s</font>" % ("#008200", parameters.strReady))
+            self.serialPortCombobox.setDisabled(True)
+            self.serailBaudrateCombobox.setDisabled(True)
+            self.serailParityCombobox.setDisabled(True)
+            self.serailStopbitsCombobox.setDisabled(True)
+            self.serailBytesCombobox.setDisabled(True)
+            self.serialOpenCloseButton.setDisabled(False)
+        else:
+            self.serialOpenCloseButton.setText(parameters.strOpen)
+            self.statusBarStauts.setText("<font color=%s>%s</font>" % ("#f31414", parameters.strClosed))
+            self.serialPortCombobox.setDisabled(False)
+            self.serailBaudrateCombobox.setDisabled(False)
+            self.serailParityCombobox.setDisabled(False)
+            self.serailStopbitsCombobox.setDisabled(False)
+            self.serailBytesCombobox.setDisabled(False)
+            self.programExitSaveParameters()
 
     def openCloseSerial(self):
         t = threading.Thread(target=self.openCloseSerialProcess)
         t.setDaemon(True)
         t.start()
-        return
+
+    def rtsChanged(self):
+        if self.checkBoxRts.isChecked():
+            self.com.setRTS(False)
+        else:
+            self.com.setRTS(True)
+    
+    def dtrChanged(self):
+        if self.checkBoxDtr.isChecked():
+            self.com.setDTR(False)
+        else:
+            self.com.setDTR(True)
 
     def portComboboxClicked(self):
         self.detectSerialPort()
-        return
 
     def getSendData(self):
         data = self.sendArea.toPlainText()
@@ -332,7 +404,7 @@ class MainWindow(QMainWindow):
                 self.errorSignal.emit( parameters.strWriteFormatError)
                 return -1
         else:
-            data = data.encode()
+            data = data.encode(self.encodingCombobox.currentText(),"ignore")
         return data
 
     def sendData(self):
@@ -359,7 +431,6 @@ class MainWindow(QMainWindow):
         except Exception as e:
             self.errorSignal.emit(parameters.strWriteError)
             print(e)
-        return
 
     def scheduledSend(self):
         self.isScheduledSending = True
@@ -370,22 +441,20 @@ class MainWindow(QMainWindow):
             except Exception:
                 self.errorSignal.emit(parameters.strTimeFormatError)
         self.isScheduledSending = False
-        return
 
     def receiveData(self):
         self.receiveProgressStop = False
         self.timeLastReceive = 0
         while(not self.receiveProgressStop):
             try:
-                length = self.com.in_waiting
-                if length>0:
-                    bytes = self.com.read(length)
+                # length = self.com.in_waiting
+                length = max(1, min(2048, self.com.in_waiting))
+                bytes = self.com.read(length)
+                if bytes!= None:
+
+                    # if self.isWaveOpen:
+                    #     self.wave.displayData(bytes)
                     self.receiveCount += len(bytes)
-                    if self.receiveSettingsHex.isChecked():
-                        strReceived = self.asciiB2HexString(bytes)
-                        self.receiveUpdateSignal.emit(strReceived)
-                    else:
-                        self.receiveUpdateSignal.emit(bytes.decode("utf-8","ignore"))
                     if self.receiveSettingsAutoLinefeed.isChecked():
                         if time.time() - self.timeLastReceive> int(self.receiveSettingsAutoLinefeedTime.text())/1000:
                             if self.sendSettingsCFLF.isChecked():
@@ -393,15 +462,19 @@ class MainWindow(QMainWindow):
                             else:
                                 self.receiveUpdateSignal.emit("\n")
                             self.timeLastReceive = time.time()
+                    if self.receiveSettingsHex.isChecked():
+                        strReceived = self.asciiB2HexString(bytes)
+                        self.receiveUpdateSignal.emit(strReceived)
+                    else:
+                        self.receiveUpdateSignal.emit(bytes.decode(self.encodingCombobox.currentText(),"ignore"))
             except Exception as e:
                 print("receiveData error")
-                if self.com.is_open and not self.serialPortCombobox.isEnabled():
-                    self.openCloseSerial()
-                    self.serialPortCombobox.clear()
-                    self.detectSerialPort()
+                # if self.com.is_open and not self.serialPortCombobox.isEnabled():
+                #     self.openCloseSerial()
+                #     self.serialPortCombobox.clear()
+                #     self.detectSerialPort()
                 print(e)
-            time.sleep(0.009)
-        return
+            # time.sleep(0.009)
 
     def updateReceivedDataDisplay(self,str):
         if str != "":
@@ -415,7 +488,6 @@ class MainWindow(QMainWindow):
                 self.receiveArea.moveCursor(QTextCursor.End)
         self.statusBarSendCount.setText("%s(bytes):%d" %(parameters.strSend ,self.sendCount))
         self.statusBarReceiveCount.setText("%s(bytes):%d" %(parameters.strReceive ,self.receiveCount))
-        return
 
     def onSendSettingsHexClicked(self):
 
@@ -423,41 +495,36 @@ class MainWindow(QMainWindow):
         data = self.asciiB2HexString(data.encode())
         self.sendArea.clear()
         self.sendArea.insertPlainText(data)
-        return
 
     def onSendSettingsAsciiClicked(self):
         try:
             data = self.sendArea.toPlainText().replace("\n"," ").strip()
             self.sendArea.clear()
             if data != "":
-                data = self.hexStringB2Hex(data).decode('utf-8','ignore')
+                data = self.hexStringB2Hex(data).decode(self.encodingCombobox.currentText(),'ignore')
                 self.sendArea.insertPlainText(data)
         except Exception as e:
-            QMessageBox.information(self,parameters.strWriteFormatError,parameters.strWriteFormatError)
-        return
+            # QMessageBox.information(self,parameters.strWriteFormatError,parameters.strWriteFormatError)
+            print("format error");
 
     def sendHistoryIndexChanged(self):
         self.sendArea.clear()
         self.sendArea.insertPlainText(self.sendHistory.currentText())
-        return
 
     def clearReceiveBuffer(self):
         self.receiveArea.clear()
         self.receiveCount = 0;
         self.sendCount = 0;
         self.receiveUpdateSignal.emit(None)
-        return
 
     def MoveToCenter(self):
         qr = self.frameGeometry()
         cp = QDesktopWidget().availableGeometry().center()
         qr.moveCenter(cp)
         self.move(qr.topLeft())
-        return
 
     def errorHint(self,str):
         QMessageBox.information(self, str, str)
-        return
 
     def closeEvent(self, event):
 
@@ -487,27 +554,30 @@ class MainWindow(QMainWindow):
             t.setDaemon(True)
             t.start()
 
+    def showCombobox(self):
+        self.serialPortCombobox.showPopup()
+
     def detectSerialPortProcess(self):
-        self.serialPortCombobox.clear()
         while(1):
-            portList = self.findSerialPort();
-            for i in portList:
-                self.serialPortCombobox.addItem(str(i[0])+" "+str(i[1]))
+            portList = self.findSerialPort()
             if len(portList)>0:
-                self.serialPortCombobox.setCurrentIndex(0)
-                self.serialPortCombobox.setToolTip(str(portList[0]))
+                currText = self.serialPortCombobox.currentText()
+                self.serialPortCombobox.clear()
+                for i in portList:
+                    showStr = str(i[0])+" "+str(i[1])
+                    self.serialPortCombobox.addItem(showStr)    
+                index = self.serialPortCombobox.findText(currText)
+                if index>=0:
+                    self.serialPortCombobox.setCurrentIndex(index)
+                else:
+                    self.serialPortCombobox.setCurrentIndex(0)
                 break
             time.sleep(1)
+        self.showSerialComboboxSignal.emit()
         self.isDetectSerialPort = False
-        return
 
     def sendHistoryFindDelete(self,str):
         self.sendHistory.removeItem(self.sendHistory.findText(str))
-        return
-
-    def test(self):
-        print("test")
-        return
 
     def asciiB2HexString(self,strB):
         strHex = binascii.b2a_hex(strB).upper()
@@ -554,22 +624,30 @@ class MainWindow(QMainWindow):
         paramObj.sendHistoryList.clear()
         for i in range(0,self.sendHistory.count()):
             paramObj.sendHistoryList.append(self.sendHistory.itemText(i))
-        f = open("settings.config","wb")
+        if self.checkBoxRts.isChecked():
+            paramObj.rts = 1
+        else:
+            paramObj.rts = 0
+        if self.checkBoxDtr.isChecked():
+            paramObj.dtr = 1
+        else:
+            paramObj.dtr = 0
+        paramObj.encodingIndex = self.encodingCombobox.currentIndex()
+        f = open("comtool.settings.config","wb")
         f.truncate()
         pickle.dump(paramObj, f)
         pickle.dump(paramObj.sendHistoryList,f)
         f.close()
-        return
 
     def programStartGetSavedParameters(self):
         paramObj = parameters.ParametersToSave()
         try:
-            f = open("settings.config", "rb")
+            f = open("comtool.settings.config", "rb")
             paramObj = pickle.load( f)
             paramObj.sendHistoryList = pickle.load(f)
             f.close()
         except Exception as e:
-            f = open("settings.config", "wb")
+            f = open("comtool.settings.config", "wb")
             f.close()
         self.serailBaudrateCombobox.setCurrentIndex(paramObj.baudRate)
         self.serailBytesCombobox.setCurrentIndex(paramObj.dataBytes)
@@ -596,8 +674,16 @@ class MainWindow(QMainWindow):
         for i in range(0, len(paramObj.sendHistoryList)):
             str = paramObj.sendHistoryList[i]
             self.sendHistory.addItem(str)
+        if paramObj.rts == 0:
+            self.checkBoxRts.setChecked(False)
+        else:
+            self.checkBoxRts.setChecked(True)
+        if paramObj.dtr == 0:
+            self.checkBoxDtr.setChecked(False)
+        else:
+            self.checkBoxDtr.setChecked(True)
+        self.encodingCombobox.setCurrentIndex(paramObj.encodingIndex)
         self.param = paramObj
-        return
 
     def keyPressEvent(self, event):
         if event.key() == Qt.Key_Control:
@@ -611,16 +697,16 @@ class MainWindow(QMainWindow):
         elif event.key() == Qt.Key_K:
             if self.keyControlPressed:
                 self.receiveArea.clear()
-        return
 
     def keyReleaseEvent(self,event):
         if event.key() == Qt.Key_Control:
             self.keyControlPressed = False
-        return
+
+    def sendAreaFontChanged(self,font):
+        print("font changed")
 
     def functionAdd(self):
         QMessageBox.information(self, "On the way", "On the way")
-        return
 
     def showHideSettings(self):
         if self.isHideSettings:
@@ -629,19 +715,16 @@ class MainWindow(QMainWindow):
         else:
             self.hideSettings()
             self.isHideSettings = True
-        return
 
     def showSettings(self):
         self.settingWidget.show()
         self.settingsButton.setStyleSheet(
             parameters.strStyleShowHideButtonLeft.replace("$DataPath",self.DataPath))
-        return;
 
     def hideSettings(self):
         self.settingWidget.hide()
         self.settingsButton.setStyleSheet(
             parameters.strStyleShowHideButtonRight.replace("$DataPath", self.DataPath))
-        return;
 
     def showHideFunctional(self):
         if self.isHideFunctinal:
@@ -650,19 +733,16 @@ class MainWindow(QMainWindow):
         else:
             self.hideFunctional()
             self.isHideFunctinal = True
-        return
 
     def showFunctional(self):
         self.functionalWiget.show()
         self.functionalButton.setStyleSheet(
             parameters.strStyleShowHideButtonRight.replace("$DataPath",self.DataPath))
-        return;
 
     def hideFunctional(self):
         self.functionalWiget.hide()
         self.functionalButton.setStyleSheet(
             parameters.strStyleShowHideButtonLeft.replace("$DataPath", self.DataPath))
-        return;
 
     def skinChange(self):
         if self.param.skin == 1: # light
@@ -672,18 +752,12 @@ class MainWindow(QMainWindow):
             file = open(self.DataPath + '/assets/qss/style.qss', "r")
             self.param.skin = 1
         self.app.setStyleSheet(file.read().replace("$DataPath", self.DataPath))
-        return
 
     def showAbout(self):
         QMessageBox.information(self, "About","<h1 style='color:#f75a5a';margin=10px;>"+parameters.appName+
                                 '</h1><br><b style="color:#08c7a1;margin = 5px;">V'+str(helpAbout.versionMajor)+"."+
                                 str(helpAbout.versionMinor)+"."+str(helpAbout.versionDev)+
                                 "</b><br><br>"+helpAbout.date+"<br><br>"+helpAbout.strAbout())
-        return
-
-    def action1(self):
-        print("action1")
-        return
 
     def autoUpdateDetect(self):
         auto = autoUpdate.AutoUpdate()
@@ -693,12 +767,22 @@ class MainWindow(QMainWindow):
     def openDevManagement(self):
         os.system('start devmgmt.msc')
 
+    # def openWaveDisplay(self):
+    #     self.wave = Wave()
+    #     self.isWaveOpen = True
+    #     self.wave.closed.connect(self.OnWaveClosed)
+    #
+    # def OnWaveClosed(self):
+    #     print("wave window closed")
+    #     self.isWaveOpen = False
+
 
 
 def main():
     app = QApplication(sys.argv)
     mainWindow = MainWindow(app)
     print("data path:"+mainWindow.DataPath)
+    print(mainWindow.param.skin)
     if(mainWindow.param.skin == 1) :# light skin
         file = open(mainWindow.DataPath+'/assets/qss/style.qss',"r")
     else: #elif mainWindow.param == 2: # dark skin
