@@ -1,11 +1,14 @@
 import sys
-from COMTool import parameters
+try:
+    import parameters
+except ImportError:
+    from COMTool import parameters
 import os
 
 versionMajor = 1
-versionMinor = 4
-versionDev   = 1
-date = "2017.8.3"
+versionMinor = 7
+versionDev   = 7
+date = "2018.12.04"
 
 def strAbout():
     pathDirList = sys.argv[0].replace("\\", "/").split("/")
